@@ -91,7 +91,22 @@ bootstrap_filter.svm <- function(object, nsim,
   out$alpha <- aperm(out$alpha, c(2, 1, 3))
   out
 }
-
+#' @method bootstrap_filter ng_ar1
+#' @export
+bootstrap_filter.ng_ar1 <- function(object, nsim,
+  seed = sample(.Machine$integer.max, size = 1), ...) {
+  
+  object$distribution <- pmatch(object$distribution, c("poisson", "binomial", "negative binomial"))
+  
+  out <- bsf(object, nsim, seed, FALSE, 4L)
+  colnames(out$at) <- colnames(out$att) <- colnames(out$Pt) <-
+    colnames(out$Ptt) <- rownames(out$Pt) <- rownames(out$Ptt) <- names(object$a1)
+  out$at <- ts(out$at, start = start(object$y), frequency = frequency(object$y))
+  out$att <- ts(out$att, start = start(object$y), frequency = frequency(object$y))
+  rownames(out$alpha) <- names(object$a1)
+  out$alpha <- aperm(out$alpha, c(2, 1, 3))
+  out
+}
 #' @method bootstrap_filter nlg_ssm
 #' @rdname bootstrap_filter
 #' @export
