@@ -1,5 +1,6 @@
 #include "ugg_ssm.h"
 #include "ugg_bsm.h"
+#include "ugg_ar1.h"
 #include "mgg_ssm.h"
 #include "lgg_ssm.h"
 
@@ -34,6 +35,10 @@ Rcpp::List gaussian_smoother(const Rcpp::List& model_, const int model_type) {
     ugg_bsm model(clone(model_), 1);
     model.smoother(alphahat, Vt);
   } break;
+  case 3: {
+    ugg_ar1 model(clone(model_), 1);
+    model.smoother(alphahat, Vt);
+  } break;
   }
   
   arma::inplace_trans(alphahat);
@@ -81,7 +86,7 @@ Rcpp::List general_gaussian_smoother(const arma::mat& y, SEXP Z, SEXP H,
     Rcpp::Named("alphahat") = alphahat,
     Rcpp::Named("Vt") = Vt);
 }
-  
+
 // [[Rcpp::export]]
 Rcpp::List gaussian_ccov_smoother(const Rcpp::List& model_, const int model_type) {
   
@@ -114,6 +119,10 @@ Rcpp::List gaussian_ccov_smoother(const Rcpp::List& model_, const int model_type
     ugg_bsm model(clone(model_), 1);
     model.smoother_ccov(alphahat, Vt, Ct);
   } break;
+  case 3: {
+    ugg_ar1 model(clone(model_), 1);
+    model.smoother_ccov(alphahat, Vt, Ct);
+  } break;
   }
   
   arma::inplace_trans(alphahat);
@@ -142,6 +151,10 @@ arma::mat gaussian_fast_smoother(const Rcpp::List& model_, const int model_type)
     ugg_bsm model(clone(model_), 1);
     return model.fast_smoother().t();
   } break;
+  case 3: {
+    ugg_ar1 model(clone(model_), 1);
+    return model.fast_smoother().t();
+  } break;
   default:
     return arma::mat(0,0);
   break;
@@ -161,6 +174,10 @@ arma::cube gaussian_sim_smoother(const Rcpp::List& model_, const unsigned int ns
     ugg_bsm model(clone(model_), seed);
     return model.simulate_states(nsim, use_antithetic);
   } break;
+  case 3: {
+    ugg_ar1 model(clone(model_), seed);
+    return model.simulate_states(nsim, use_antithetic);
+  } break;
   default:
     return arma::cube(0,0,0);
   break;
@@ -192,5 +209,10 @@ arma::cube general_gaussian_sim_smoother(const arma::mat& y, SEXP Z, SEXP H,
     time_varying, n_states, n_etas, seed);
   mgg_ssm mgg_model = model.build_mgg();
   
-  return mgg_model.simulate_states();
+  arma::cube asim(model.m, model.n + 1, nsim);
+  for (unsigned int i = 0; i < nsim; i++) {
+    asim.slice(i) = mgg_model.simulate_states();
+  }
+  
+  return asim;
 }
