@@ -13,31 +13,37 @@
 #include "rep_mat.h"
 #include "distr_consts.h"
 #include "filter_smoother.h"
+#include "summary.h"
 
 ung_amcmc::ung_amcmc(const unsigned int n_iter, 
   const unsigned int n_burnin, const unsigned int n_thin, const unsigned int n, 
   const unsigned int m, const double target_acceptance, const double gamma, 
-  const arma::mat& S, const bool store_states) :
+  const arma::mat& S, const unsigned int output_type, const bool store_modes) :
   mcmc(n_iter, n_burnin, n_thin, n, m,
-    target_acceptance, gamma, S, store_states),
+    target_acceptance, gamma, S, output_type),
     weight_storage(arma::vec(n_samples, arma::fill::zeros)),
-    y_storage(arma::mat(n, n_samples)), H_storage(arma::mat(n, n_samples)),
-    scales_storage(arma::mat(n, n_samples)),
+    y_storage(arma::mat(n, n_samples * store_modes)), 
+    H_storage(arma::mat(n, n_samples * store_modes)),
+    scales_storage(arma::mat(n, n_samples * store_modes)),
     approx_loglik_storage(arma::vec(n_samples)), 
-    prior_storage(arma::vec(n_samples)){
+    prior_storage(arma::vec(n_samples)), store_modes(store_modes) {
 }
 
 void ung_amcmc::trim_storage() {
   theta_storage.resize(n_par, n_stored);
   posterior_storage.resize(n_stored);
   count_storage.resize(n_stored);
-  alpha_storage.resize(alpha_storage.n_rows, alpha_storage.n_cols, n_stored);
-  weight_storage.resize(n_stored);
-  scales_storage.resize(y_storage.n_rows, n_stored);
-  y_storage.resize(y_storage.n_rows, n_stored);
-  H_storage.resize(H_storage.n_rows, n_stored);
+  if (output_type == 1) {
+    alpha_storage.resize(alpha_storage.n_rows, alpha_storage.n_cols, n_stored);
+  }
   approx_loglik_storage.resize(n_stored);
+  weight_storage.resize(n_stored);
   prior_storage.resize(n_stored);
+  if (store_modes) {
+    scales_storage.resize(y_storage.n_rows, n_stored);
+    y_storage.resize(y_storage.n_rows, n_stored);
+    H_storage.resize(H_storage.n_rows, n_stored);
+  }
 }
 
 void ung_amcmc::expand() {
@@ -53,10 +59,28 @@ void ung_amcmc::expand() {
   posterior_storage.set_size(n_stored);
   posterior_storage = expanded_posterior;
   
-  arma::cube expanded_alpha = rep_cube(alpha_storage, count_storage);
-  alpha_storage.set_size(alpha_storage.n_rows, alpha_storage.n_cols, n_stored);
-  alpha_storage = expanded_alpha;
+  arma::vec expanded_weight = rep_vec(weight_storage, count_storage);
+  weight_storage.set_size(n_stored);
+  weight_storage = expanded_weight;
   
+  arma::vec expanded_prior = rep_vec(prior_storage, count_storage);
+  prior_storage.set_size(n_stored);
+  prior_storage = expanded_prior;
+  
+  count_storage.resize(n_stored);
+  count_storage.ones();
+  
+  arma::vec expanded_approx_loglik = rep_vec(approx_loglik_storage, count_storage);
+  approx_loglik_storage.set_size(n_stored);
+  approx_loglik_storage = expanded_approx_loglik;
+  
+  if (output_type == 1) {
+    arma::cube expanded_alpha = rep_cube(alpha_storage, count_storage);
+    alpha_storage.set_size(alpha_storage.n_rows, alpha_storage.n_cols, n_stored);
+    alpha_storage = expanded_alpha;
+  }
+  
+  if (store_modes) {
   arma::mat expanded_scales = rep_mat(scales_storage, count_storage);
   scales_storage.set_size(scales_storage.n_rows, n_stored);
   scales_storage = expanded_scales;
@@ -69,21 +93,7 @@ void ung_amcmc::expand() {
   H_storage.set_size(H_storage.n_rows, n_stored);
   H_storage = expanded_H;
   
-  arma::vec expanded_weight = rep_vec(weight_storage, count_storage);
-  weight_storage.set_size(n_stored);
-  weight_storage = expanded_weight;
-  
-  arma::vec expanded_approx_loglik = rep_vec(approx_loglik_storage, count_storage);
-  approx_loglik_storage.set_size(n_stored);
-  approx_loglik_storage = expanded_approx_loglik;
-  
-  arma::vec expanded_prior = rep_vec(prior_storage, count_storage);
-  prior_storage.set_size(n_stored);
-  prior_storage = expanded_prior;
-  
-  count_storage.resize(n_stored);
-  count_storage.ones();
-  
+  }
 }
 
 // run approximate MCMC for
@@ -100,7 +110,7 @@ template void ung_amcmc::approx_mcmc(ung_svm model, const bool end_ram,
 template void ung_amcmc::approx_mcmc(ung_ar1 model, const bool end_ram,
   const bool local_approx, const arma::vec& initial_mode,
   const unsigned int max_iter, const double conv_tol);
-  
+
 template<class T>
 void ung_amcmc::approx_mcmc(T model, const bool end_ram, const bool local_approx,
   const arma::vec& initial_mode, const unsigned int max_iter, const double conv_tol) {
@@ -202,10 +212,12 @@ void ung_amcmc::approx_mcmc(T model, const bool end_ram, const bool local_approx
       if (new_value) {
         approx_loglik_storage(n_stored) = approx_loglik;
         theta_storage.col(n_stored) = theta;
-        y_storage.col(n_stored) = approx_y;
-        H_storage.col(n_stored) = approx_H;
+        if (store_modes) {
+          y_storage.col(n_stored) = approx_y;
+          H_storage.col(n_stored) = approx_H;
+          scales_storage.col(n_stored) = scales;
+        }
         prior_storage(n_stored) = logprior;
-        scales_storage.col(n_stored) = scales;
         count_storage(n_stored) = 1;
         n_stored++;
         new_value = false;
@@ -234,13 +246,16 @@ template void ung_amcmc::is_correction_psi(ung_svm model, const unsigned int nsi
   const unsigned int is_type, const unsigned int n_threads);
 template void ung_amcmc::is_correction_psi(ung_ar1 model, const unsigned int nsim_states, 
   const unsigned int is_type, const unsigned int n_threads);
-  
+
 template <class T>
 void ung_amcmc::is_correction_psi(T model, const unsigned int nsim_states, 
   const unsigned int is_type, const unsigned int n_threads) {
   
+  arma::cube Valpha(model.m, model.m, model.n + 1, arma::fill::zeros);
+  double sum_w = 0.0;
+  
 #ifdef _OPENMP
-#pragma omp parallel num_threads(n_threads) default(none) firstprivate(model) 
+#pragma omp parallel num_threads(n_threads) default(none) shared(Valpha, sum_w) firstprivate(model) 
 {
   
   model.engine = sitmo::prng_engine(omp_get_thread_num() + 1);
@@ -278,19 +293,31 @@ void ung_amcmc::is_correction_psi(T model, const unsigned int nsim_states,
     double loglik = model.psi_filter(approx_model, 0, scales_storage.col(i),
       nsim, alpha_i, weights_i, indices);
     
-    if(arma::is_finite(loglik)) {
-      weight_storage(i) = std::exp(loglik);
+    weight_storage(i) = std::exp(loglik);
+    if (output_type != 3) {
       filter_smoother(alpha_i, indices);
       arma::vec w = weights_i.col(model.n);
-      std::discrete_distribution<unsigned int> sample(w.begin(), w.end());
-      alpha_storage.slice(i) = alpha_i.slice(sample(model.engine)).t();
-    } else {
-      weight_storage(i) = 0.0;
-      alpha_storage.slice(i).zeros();
+      if (output_type == 1) {
+        std::discrete_distribution<unsigned int> sample(w.begin(), w.end());
+        alpha_storage.slice(i) = alpha_i.slice(sample(model.engine)).t();
+      } else {
+        arma::mat alphahat_i(model.m, model.n + 1);
+        arma::cube Vt_i(model.m, model.m, model.n + 1);
+        weighted_summary(alpha_i, alphahat_i, Vt_i, w);
+#pragma omp critical
+{
+  arma::mat diff = alphahat_i - alphahat;
+  double tmp = count_storage(i) + sum_w;
+  alphahat = (alphahat * sum_w + alphahat_i * count_storage(i)) / tmp;
+  for (unsigned int t = 0; t < model.n + 1; t++) {
+    Valpha.slice(t) += diff.col(t) * (alphahat_i.col(t) - alphahat.col(t)).t();
+  }
+  Vt = (Vt * sum_w + Vt_i * count_storage(i)) / tmp;
+  sum_w = tmp;
+}
+      }
     }
-    
   }
-  
 }
 #else
 arma::vec tmp(1);
@@ -324,20 +351,34 @@ for (unsigned int i = 0; i < theta_storage.n_cols; i++) {
   double loglik = model.psi_filter(approx_model, 0, scales_storage.col(i),
     nsim, alpha_i, weights_i, indices);
   
-  if(arma::is_finite(loglik)) {
-    weight_storage(i) = std::exp(loglik);
+  weight_storage(i) = std::exp(loglik);
+  if (output_type != 3) {
     filter_smoother(alpha_i, indices);
     arma::vec w = weights_i.col(model.n);
-    std::discrete_distribution<unsigned int> sample(w.begin(), w.end());
-    alpha_storage.slice(i) = alpha_i.slice(sample(model.engine)).t();
-  } else {
-    weight_storage(i) = 0.0;
-    alpha_storage.slice(i).zeros();
+    if (output_type == 1) {
+      std::discrete_distribution<unsigned int> sample(w.begin(), w.end());
+      alpha_storage.slice(i) = alpha_i.slice(sample(model.engine)).t();
+    } else {
+      arma::mat alphahat_i(model.m, model.n + 1);
+      arma::cube Vt_i(model.m, model.m, model.n + 1);
+      weighted_summary(alpha_i, alphahat_i, Vt_i, w);
+      
+      arma::mat diff = alphahat_i - alphahat;
+      double tmp = count_storage(i) + sum_w;
+      alphahat = (alphahat * sum_w + alphahat_i * count_storage(i)) / tmp;
+      for (unsigned int t = 0; t < model.n + 1; t++) {
+        Valpha.slice(t) += diff.col(t) * (alphahat_i.col(t) - alphahat.col(t)).t();
+      }
+      Vt = (Vt * sum_w + Vt_i * count_storage(i)) / tmp;
+      sum_w = tmp;
+    }
   }
 }
 
 #endif
-
+if (output_type == 2) {
+  Vt += Valpha / theta_storage.n_cols; // Var[E(alpha)] + E[Var(alpha)]
+}
 posterior_storage = prior_storage + approx_loglik_storage + 
   arma::log(weight_storage);
 }
@@ -359,8 +400,11 @@ template <class T>
 void ung_amcmc::is_correction_bsf(T model, const unsigned int nsim_states, 
   const unsigned int is_type, const unsigned int n_threads) {
   
+  arma::cube Valpha(model.m, model.m, model.n + 1, arma::fill::zeros);
+  double sum_w = 0.0;
+  
 #ifdef _OPENMP
-#pragma omp parallel num_threads(n_threads) default(none) firstprivate(model) 
+#pragma omp parallel num_threads(n_threads) default(none) shared(Valpha, sum_w) firstprivate(model) 
 {
   
   model.engine = sitmo::prng_engine(omp_get_thread_num() + 1);
@@ -380,16 +424,31 @@ void ung_amcmc::is_correction_bsf(T model, const unsigned int nsim_states,
     arma::umat indices(nsim, model.n);
     
     double loglik = model.bsf_filter(nsim, alpha_i, weights_i, indices);
-    if(arma::is_finite(loglik)) {
-      weight_storage(i) = std::exp(loglik - approx_loglik_storage(i));
+    weight_storage(i) = std::exp(loglik - approx_loglik_storage(i));
+    if (output_type != 3) {
       filter_smoother(alpha_i, indices);
       arma::vec w = weights_i.col(model.n);
-      std::discrete_distribution<unsigned int> sample(w.begin(), w.end());
-      alpha_storage.slice(i) = alpha_i.slice(sample(model.engine)).t();
-    } else {
-      weight_storage(i) = 0.0;
-      alpha_storage.slice(i).zeros();
+      if (output_type == 1) {
+        std::discrete_distribution<unsigned int> sample(w.begin(), w.end());
+        alpha_storage.slice(i) = alpha_i.slice(sample(model.engine)).t();
+      } else {
+        arma::mat alphahat_i(model.m, model.n + 1);
+        arma::cube Vt_i(model.m, model.m, model.n + 1);
+        weighted_summary(alpha_i, alphahat_i, Vt_i, w);
+#pragma omp critical
+{
+  arma::mat diff = alphahat_i - alphahat;
+  double tmp = count_storage(i) + sum_w;
+  alphahat = (alphahat * sum_w + alphahat_i * count_storage(i)) / tmp;
+  for (unsigned int t = 0; t < model.n + 1; t++) {
+    Valpha.slice(t) += diff.col(t) * (alphahat_i.col(t) - alphahat.col(t)).t();
+  }
+  Vt = (Vt * sum_w + Vt_i * count_storage(i)) / tmp;
+  sum_w = tmp;
+}
+      }
     }
+    
   }
 }
 #else
@@ -407,18 +466,34 @@ for (unsigned int i = 0; i < theta_storage.n_cols; i++) {
   arma::umat indices(nsim, model.n);
   
   double loglik = model.bsf_filter(nsim, alpha_i, weights_i, indices);
-  if(arma::is_finite(loglik)) {
-    weight_storage(i) = std::exp(loglik - approx_loglik_storage(i));
+  weight_storage(i) = std::exp(loglik - approx_loglik_storage(i));
+  
+  if (output_type != 3) {
     filter_smoother(alpha_i, indices);
     arma::vec w = weights_i.col(model.n);
-    std::discrete_distribution<unsigned int> sample(w.begin(), w.end());
-    alpha_storage.slice(i) = alpha_i.slice(sample(model.engine)).t();
-  } else {
-    weight_storage(i) = 0.0;
-    alpha_storage.slice(i).zeros();
+    if (output_type == 1) {
+      std::discrete_distribution<unsigned int> sample(w.begin(), w.end());
+      alpha_storage.slice(i) = alpha_i.slice(sample(model.engine)).t();
+    } else {
+      arma::mat alphahat_i(model.m, model.n + 1);
+      arma::cube Vt_i(model.m, model.m, model.n + 1);
+      weighted_summary(alpha_i, alphahat_i, Vt_i, w);
+      arma::mat diff = alphahat_i - alphahat;
+      double tmp = count_storage(i) + sum_w;
+      alphahat = (alphahat * sum_w + alphahat_i * count_storage(i)) / tmp;
+      for (unsigned int t = 0; t < model.n + 1; t++) {
+        Valpha.slice(t) += diff.col(t) * (alphahat_i.col(t) - alphahat.col(t)).t();
+      }
+      Vt = (Vt * sum_w + Vt_i * count_storage(i)) / tmp;
+      sum_w = tmp;
+      
+    }
   }
 }
 #endif
+if (output_type == 2) {
+  Vt += Valpha / theta_storage.n_cols; // Var[E(alpha)] + E[Var(alpha)]
+}
 posterior_storage = prior_storage + arma::log(weight_storage);
 }
 
@@ -435,9 +510,11 @@ template <class T>
 void ung_amcmc::is_correction_spdk(T model, const unsigned int nsim_states, 
   const unsigned int is_type, const unsigned int n_threads) {
   
+  arma::cube Valpha(model.m, model.m, model.n + 1, arma::fill::zeros);
+  double sum_w = 0.0;
   
 #ifdef _OPENMP
-#pragma omp parallel num_threads(n_threads) default(none) firstprivate(model) 
+#pragma omp parallel num_threads(n_threads) default(none) shared(Valpha, sum_w) firstprivate(model) 
 {
   
   model.engine = sitmo::prng_engine(omp_get_thread_num() + 1);
@@ -472,9 +549,27 @@ void ung_amcmc::is_correction_spdk(T model, const unsigned int nsim_states,
     arma::vec weights_i = model.importance_weights(approx_model, alpha_i);
     weights_i = arma::exp(weights_i - arma::accu(scales_storage.col(i)));
     weight_storage(i) = arma::mean(weights_i);
-    std::discrete_distribution<unsigned int> sample(weights_i.begin(), weights_i.end());
-    alpha_storage.slice(i) = alpha_i.slice(sample(model.engine)).t();
-    
+    if (output_type != 3) {
+      if (output_type == 1) {
+        std::discrete_distribution<unsigned int> sample(weights_i.begin(), weights_i.end());
+        alpha_storage.slice(i) = alpha_i.slice(sample(model.engine)).t();
+      } else {
+        arma::mat alphahat_i(model.m, model.n + 1);
+        arma::cube Vt_i(model.m, model.m, model.n + 1);
+        weighted_summary(alpha_i, alphahat_i, Vt_i, weights_i);
+#pragma omp critical
+{
+  arma::mat diff = alphahat_i - alphahat;
+  double tmp = count_storage(i) + sum_w;
+  alphahat = (alphahat * sum_w + alphahat_i * count_storage(i)) / tmp;
+  for (unsigned int t = 0; t < model.n + 1; t++) {
+    Valpha.slice(t) += diff.col(t) * (alphahat_i.col(t) - alphahat.col(t)).t();
+  }
+  Vt = (Vt * sum_w + Vt_i * count_storage(i)) / tmp;
+  sum_w = tmp;
+}
+      }
+    }
   }
   
 }
@@ -507,13 +602,30 @@ for (unsigned int i = 0; i < theta_storage.n_cols; i++) {
   arma::cube alpha_i = approx_model.simulate_states(nsim, true);
   arma::vec weights_i = model.importance_weights(approx_model, alpha_i);
   weights_i = arma::exp(weights_i - arma::accu(scales_storage.col(i)));
-  weight_storage(i) = arma::mean(weights_i);
-  std::discrete_distribution<unsigned int> sample(weights_i.begin(), weights_i.end());
-  alpha_storage.slice(i) = alpha_i.slice(sample(model.engine)).t();
+  if (output_type != 3) {
+    if (output_type == 1) {
+      std::discrete_distribution<unsigned int> sample(weights_i.begin(), weights_i.end());
+      alpha_storage.slice(i) = alpha_i.slice(sample(model.engine)).t();
+    } else {
+      arma::mat alphahat_i(model.m, model.n + 1);
+      arma::cube Vt_i(model.m, model.m, model.n + 1);
+      weighted_summary(alpha_i, alphahat_i, Vt_i, weights_i);
+      arma::mat diff = alphahat_i - alphahat;
+      double tmp = count_storage(i) + sum_w;
+      alphahat = (alphahat * sum_w + alphahat_i * count_storage(i)) / tmp;
+      for (unsigned int t = 0; t < model.n + 1; t++) {
+        Valpha.slice(t) += diff.col(t) * (alphahat_i.col(t) - alphahat.col(t)).t();
+      }
+      Vt = (Vt * sum_w + Vt_i * count_storage(i)) / tmp;
+      sum_w = tmp;
+    }
+  }
 }
 
 #endif
-
+if (output_type == 2) {
+  Vt += Valpha / theta_storage.n_cols; // Var[E(alpha)] + E[Var(alpha)]
+}
 posterior_storage = prior_storage + approx_loglik_storage + 
   arma::log(weight_storage);
 }
@@ -528,7 +640,7 @@ void ung_amcmc::approx_state_posterior(T model, const unsigned int n_threads) {
   
 #ifdef _OPENMP
 #pragma omp parallel num_threads(n_threads) default(none) firstprivate(model) 
-  {
+{
   
   model.engine = sitmo::prng_engine(omp_get_thread_num() + 1);
   
